# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'

Puppet::Type.type(:db2_validation).provide(:db2) do
  include EasyType::Provider
  include PuppetX::EnterpriseModules::Db2::Commands

  confine :feature => :db2_running

  desc 'Verify db2 settings'

  mk_resource_methods

  def execute_check
    records = execute_query(resource.check)
    @records = records.collect do |r|
      r = r.transform_keys(&:downcase).transform_keys(&:to_sym)
      r.transform_values(&:strip)
    end
    @records
  end

  def execute_query(query)
    options = { :instance => resource.instance_name, :database => resource.database_name, :parse => true }
    options[:username] = username if resource.username.to_s != 'absent'
    options[:password] = password if resource.password.to_s != 'absent'
    sql query, options
  end

  def report
    message = if resource.fail_message == :absent
                "Validation #{name}  failed; expected: #{resource.expected_number_of_rows} rows got : #{@records.count}"
              elsif resource.fail_message.include?('%')
                @records.collect { |r| format(resource.fail_message, r) }
              else
                resource.fail_message
              end
    message = "#{resource.path}: #{message}"

    case resource['report_as']
    when /warning/i
      Puppet.warning message
    when /error/i
      raise Puppet::Error, message
    when /command/i
      # Issue a waning and execute the fail_command
      Puppet.warning message
      final_command = format(fail_command, :messages => @messages.join(';'))
      Puppet.debug Puppet::Util::Execution.execute(
        final_command,
        :failonfail => true,
        :combine => true,
        :override_locale => false
      )
    when /none/i
      Puppet.debug message
    else
      raise "#{report_as} is not a valid value"
    end
  end
end
