# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/core'

Puppet::Type.newtype(:db2_validation) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Type
  include PuppetX::EnterpriseModules::Db2::Access

  desc <<-DESC
  This type allows you to validate the contents of your DB2 database. Here is an example on how you can
  use this:

      db2_validation { "db2inst1/MYDB/No user data in system tablespaces":
        check                   => "select tabschema,tabname,tbspace from syscat.tables where tabschema not in ('ADMINISTRATOR','SYSIBM','SYSTOOLS') and tbspace in ('SYSCATSPACE','SYSTOOLSPACE','SYSTOOLSTMPSPACE','TEMPSPACE')",
        expected_number_of_rows => 0,
        fail_message            => 'User table %<tabschema>s.%<tabname>s not allowed system tablespace %<tbspace>s',
        report_as               => 'command',
        fail_command            => "/bin/send_to_montoring --node ${::fqdn} %<output>s",
      }

  The Puppet statement executes a query on the database and report a warning when more the 0 records are
  found. It uses the returned records of the query to generate a good reading error or warning message.
  DESC

  map_titles_to_attributes([
                             %r{^((.*)/(.*)/(.*))$}, [:name, :instance_name, :database_name, :check],
                             %r{^((.*)/(.*))$},      [:name, :instance_name, :database_name]
                           ])

  to_get_raw_resources do
    []
  end
  #
  # Register the result(s) of a control in the class. The puppet type validation_report can
  # then pass it to the puppet server or any oher place.
  #
  def self.register_result(qualified_control, result, message)
    return if qualified_control.nil?

    control, target = qualified_control.split(/\[|\]/)
    @results ||= {}
    #
    # SOmetimes insync is called multiple times. To guard aganst double messages, check if the message is already registered.
    #
    return if @results[qualified_control] && @results[qualified_control]['results'].any? { |e| e[:message] == message }

    @results[qualified_control] ||= {}
    @results[qualified_control]['control'] ||= control
    @results[qualified_control]['target'] ||= target
    @results[qualified_control]['results'] ||= []
    @results[qualified_control]['results'] << { :message => message, :result => result }
  end

  class << self
    attr_reader :results
  end

  on_modify do
    provider.report
  end

  def refresh
    records = provider.execute_check

    provider.report if records.count != expected_number_of_rows
  end

  parameter :name
  property  :check
  parameter :instance_name
  parameter :database_name
  parameter :refreshonly
  parameter :control
  parameter :skip

  parameter :timeout
  parameter :password
  parameter :username
  parameter :report_as
  parameter :expected_number_of_rows
  parameter :fail_message
  parameter :fail_command
end
