# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'tempfile'
require 'fileutils'
require 'puppet_x/enterprisemodules/db2/commands'
require 'puppet_x/enterprisemodules/db2/type'

module PuppetX
  module EnterpriseModules
    module Db2
      # Docs
      module Access
        include ::EasyType::Helpers
        include PuppetX::EnterpriseModules::Db2::Commands
        include PuppetX::EnterpriseModules::Db2::Type

        def module_name
          'db2_config'
        end

        def self.included(parent)
          parent.extend(Access)
        end

        def for_version(hash)
          hash.collect do |version, query|
            [lambda { |instance| db2_version?(instance, version) }, query]
          end
        end

        #
        # Check if the Db2 version of the specified instance is equal to the
        # specified version.
        #
        def db2_version?(instance, version)
          return true if version.to_s == 'default'

          if version.instance_of?(String) || version.instance_of?(Symbol)
            db2_major_version(instance) == version.to_s.to_i
          elsif version.instance_of?(Regexp)
            database_version(instance) =~ version
          else
            fail 'Invalid type specified for sql version query'
          end
        end

        ##
        #
        # Use this function to execute Db2 statements on all running instances.
        #
        # @param query [String] this is the querys to be given
        #
        #
        def sql_on_all_databases(query, _parameters = {})
          data = []
          db2_instance_names.each do |instance|
            database_names_on(instance).each do |database|
              data += sql(query, :instance => instance, :database => database, :parse => true)
            end
          end
          data
        end

        ##
        #
        # Use this function to execute Db2 statements on all running instances.
        #
        # @param query [String] this is the querys to be given
        #
        #
        def sql_on_all_primary_databases(query, _parameters = {})
          data = []
          db2_instance_names.each do |instance|
            primary_database_names_on(instance).each do |database|
              data += sql(query, :instance => instance, :database => database, :parse => true)
            end
          end
          data
        end

        # @private
        #
        # Returns the first valid sql statement in the versioned command
        #
        def select_statement(versioned_command, instance)
          versioned_command.each do |version_proc, statement|
            return statement if version_proc.call(instance)
          end
          fail 'no valid version found'
        end

        # @private
        def versioned_statement?(command)
          command.is_a?(::Array)
        end

        def add_instance_to(elements, instance)
          elements.collect do |e|
            e['instance'] = instance
            e
          end
        end

        # This is a little hack to get a specified timeout value
        def timeout_specified
          resource.timeout
        rescue NameError
          nil
        end

        def instance_from_resource
          instance_from(resource)
        end
      end
    end
  end
end
