# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'tmpdir'
module PuppetX
  module EnterpriseModules
    module Db2
      # docs
      class Command
        DEFAULT_TIMEOUT ||= 300 # 5 minutes

        # rubocop: disable Style/ClassVars
        @@created_files ||= []
        # rubocop: enable Style/ClassVars

        COMMAND_VALID_OPTIONS ||= [
          :instance,
          :database,
          :schema,
          :username,
          :password,
          :timeout
        ].freeze

        attr_reader(*COMMAND_VALID_OPTIONS)

        def initialize(command, options, valid_options = COMMAND_VALID_OPTIONS)
          @valid_options = valid_options
          check_options(options)
          @command  = command
          @timeout  = options.fetch(:timeout) { DEFAULT_TIMEOUT }
          @instance = options.fetch(:instance) { raise ArgumentError, 'you need to specify a instance for Db2 access' }
          @database = options.fetch(:database, nil)
          @schema   = options.fetch(:schema, nil)
          @username = options.fetch(:username, nil)
          @password = options[:password] # may be empty
        end

        def command_string(arguments = '')
          # rubocop: disable Style/GuardClause
          if Puppet::Util::Platform.windows?
            fail 'No windows support yet'
          else
            "su - #{@instance} -c \"cd #{working_dir}; #{@command} #{arguments}\""
          end
          # rubocop: enable Style/GuardClause
        end

        def execute(arguments)
          options = { :failonfail => true, :combine => true }
          value = ''
          within_time(@timeout) do
            Puppet.debug "command.rb: Executing #{@command} command: #{arguments} on #{@instance}"
            value = if Puppet::Util::Platform.windows?
                      `command_string(arguments)`
                    else
                      Puppet::Util::Execution.execute(command_string(arguments), options)
                    end
          end
          value
        end

        protected

        def working_dir
          pwd = Dir.pwd
          unless writable_by_user(@instance, pwd)
            pwd = Dir.tmpdir
            Puppet.debug "command.rb: Running Db2 command from a pwd that is not writable for instance #{instance} therefore resetting pwd to '#{pwd}'"
          end
          pwd
        end

        private

        def writable_by_user(instance, pwd)
          # TODO: what mechanisme do we need for Windows
          if Puppet::Util::Platform.windows?
            true
          else
            system "su - #{instance} -c \"cd #{pwd};\" > /dev/null 2>&1"
          end
        end

        def within_time(timeout, &block)
          Puppet.debug "command.rb: Using timeout #{timeout}"
          if timeout.zero?
            yield
          else
            Timeout.timeout(timeout, &block)
          end
        end

        def check_options(options)
          options.each_key { |key| raise ArgumentError, "option #{key} invalid for #{@command}. Only #{@valid_options.join(', ')} are supported" unless @valid_options.include?(key) }
        end
      end
    end
  end
end
