# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/db2/command'
require 'tmpdir'

module PuppetX
  module EnterpriseModules
    module Db2
      # docs
      class Db2Command < PuppetX::EnterpriseModules::Db2::Command
        include ::EasyType::Template

        DB2_VALID_OPTIONS ||= [
          :instance,
          :database,
          :schema,
          :username,
          :password,
          :timeout,
          :parse,
          :failonfail
        ].freeze

        def initialize(options = {})
          super(:db2, options, DB2_VALID_OPTIONS)
          @parse      = options.fetch(:parse, nil)
          @failonfail = options.fetch(:failonfail, true)
        end

        def execute(arguments)
          options = { :failonfail => @failonfail, :combine => true }
          value = ''
          within_time(@timeout) do
            value = if @database && @parse
                      export_from_database(arguments, options)
                    elsif @database
                      execute_on_database(arguments, options)
                    else
                      execute_on_instance(arguments, options)
                    end
          end
          value
        end

        def execute_on_database(arguments, options)
          Puppet.debug "db2_command.rb: Executing #{@command} command: #{arguments} on #{@instance} and database #{@database}"
          script = command_file(template('db2_config/db2_config/execute.db2.erb', binding))
          # rubocop: disable Style/GuardClause
          if Puppet::Util::Platform.windows?
            fail 'No windows support yet'
          else
            Puppet::Util::Execution.execute(command_string("-ntf #{script}"), options)
          end
          # rubocop: enable Style/GuardClause
        end

        def export_from_database(arguments, options)
          Puppet.debug "db2_command.rb: Executing #{@command} command: export #{arguments} on #{@instance} and database #{@database}"
          # rubocop: disable Lint/EmptyBlock
          output_file = Dir::Tmpname.create(['prefix-', '.ext'], "/home/#{@instance}") {}
          # rubocop: enable Lint/EmptyBlock
          script = command_file(template('db2_config/db2_config/external.db2.erb', binding))
          # rubocop: disable Style/GuardClause
          if Puppet::Util::Platform.windows?
            fail 'No windows support yet'
          else
            Puppet::Util::Execution.execute(command_string("-ntf #{script}"), options)
          end
          # rubocop: enable Style/GuardClause
          return [] unless File.exist?(output_file)

          results = File.read(output_file)
          File.delete(output_file)
          results.encode!('UTF-8', :invalid => :replace, :undef => :replace)
          add_info(convert_csv_data_to_hash(results, [], :col_sep => ',', :converters => lambda { |f| f ? f.strip : nil }))
        end

        def execute_on_instance(arguments, options)
          Puppet.debug "db2_command.rb: Executing #{@command} command: #{arguments} on #{@instance}, connected as #{@username}"
          # rubocop: disable Style/GuardClause
          if Puppet::Util::Platform.windows?
            fail 'No windows support yet'
          else
            Puppet::Util::Execution.execute(command_string(arguments), options)
          end
          # rubocop: enable Style/GuardClause
        end

        def add_info(elements)
          elements.collect do |e|
            e[:database] = @database
            e[:instance] = @instance
            e
          end
        end

        def command_file(content, type = '.db2')
          command_file = Tempfile.new(['command', type])
          ObjectSpace.undefine_finalizer(command_file) # Don't delete the file
          command_file.write(content)
          command_file.close
          FileUtils.chown(@instance, nil, command_file.path)
          FileUtils.chmod(0o600, command_file.path)
          FileUtils.chown(@instance, nil, command_file.path)
          FileUtils.chmod(0o600, command_file.path)
          path = command_file.path
          @@created_files << path
          path
        end
      end
    end
  end
end
