# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/commands'
require 'puppet_x/enterprisemodules/db2/db2_command'
require 'puppet_x/enterprisemodules/db2/db2cm_parser'

# rubocop: disable Style/ClassVars
module PuppetX
  module EnterpriseModules
    module Db2
      #
      # Define methods common for DB2 types
      #
      module Type
        def self.included(parent)
          parent.send(:include, ::EasyType)
          parent.send(:include, PuppetX::EnterpriseModules::Db2::Commands)
          parent.extend ClassMethods
        end

        def primary?(instance, database)
          self.class.primary_database_names_on(instance).include?(database)
        end

        def when_primary
          if primary?(instance_name, database_name)
            yield
          else
            Puppet.notice "#{path}: Skipping DB2 query because on database #{database_name} on instance #{instance_name} is running in standby mode."
            nil
          end
        end

        def actual_location
          location.nil? ? self.class.db2_locations.first : location
        end

        # Class methods
        module ClassMethods
          @@dbsls = nil
          @@db2_instances = nil
          @@databases_on = nil

          def db2ls
            @@db2ls ||= Puppet::Util::Execution.execute('/usr/local/bin/db2ls -c') || ''
          end

          def db2_locations
            data_lines = db2ls.split("\n").drop(1)
            data_lines.collect do |line|
              line.split(':')[0]
            end
          end

          def pcmkr_clusters
            PuppetX::EnterpriseModules::Db2::Db2cmParser.instance.clusters
          end

          def pcmkr_instances
            PuppetX::EnterpriseModules::Db2::Db2cmParser.instance.instances
          end

          def pcmkr_databases
            PuppetX::EnterpriseModules::Db2::Db2cmParser.instance.databases
          end

          def pcmkr_vips
            PuppetX::EnterpriseModules::Db2::Db2cmParser.instance.vips
          end

          def pcmkr_qdevices
            PuppetX::EnterpriseModules::Db2::Db2cmParser.instance.qdevices
          end

          def db2_instance_cache_reset
            @@db2_instances = nil
          end

          def db2_instances
            return @@db2_instances if @@db2_instances

            basic_instance_data = db2_locations.reduce([]) do |value, location|
              value += Puppet::Util::Execution.execute("#{location}/bin/db2ilist").split("\n")
              value
            end
            instance_data = basic_instance_data.collect do |instance|
              raw_data = db2('get database manager configuration', :instance => instance, :failonfail => true)
              [:name, instance] + raw_data.scan(/\s+(.*?)\s+=(.*)$/).flatten
            end
            @@db2_instances = instance_data.collect { |e| ::EasyType::Helpers::InstancesResults[*e] }
          end

          def db2_instance_names
            db2_instances.collect { |e| e[:name] }
          end

          def databases_on_cache_reset(instance)
            @@databases_on[instance] = nil
          end

          #
          # Returns a Hash containing the basic database information
          #
          def databases_on(instance)
            @@databases_on ||= {}
            return @@databases_on[instance] if @@databases_on[instance]

            raw_data = db2('list database directory', :instance => instance, :failonfail => false)
            return [] if raw_data.include?('SQL1057W') # Intial case no database available

            data = raw_data.split('entry:').drop(1)

            @@databases_on[instance] = data.collect do |db_summary|
              basic_info = ::EasyType::Helpers::InstancesResults[db_summary.scan(/\s+(.*?)\s+=(.*)$/)]
              db_name = basic_info['Database name'].strip
              begin
                hadr_raw = db2pd("-hadr  -db #{db_name}", :instance => instance)
                hadr_info = ::EasyType::Helpers::InstancesResults[hadr_raw.scan(/\s+(.*?)\s+=(.*)$/)]
                basic_info.merge!(hadr_info)
              rescue Puppet::ExecutionFailure => e
                #
                # On non hadr databases this will fail, so we need to guard against this
                #
                raise unless e.message.include?('not activated on database member 0')

                Puppet.debug "HADR not activated on database #{db_name}"
              end
              basic_info
            end
          end

          #
          # Returns a Hash containing the basic database information
          #
          def database_names_on(instance)
            databases_on(instance).collect { |e| e['Database name'].strip }
          end

          #
          # Returns a Hash containing the basic database information. When in hadr mode, select
          #  only databases who's hadr role is primary. WHen the hadr role is not set, we are **NOT**
          # running in hadr and all databases are primary databases.
          #
          def primary_database_names_on(instance)
            databases_on(instance).select { |e| e['HADR_ROLE']&.strip == 'PRIMARY' || e['HADR_ROLE'].nil? }.collect { |e| e['Database name'].strip }
          end

          #
          # Returns a Hash containing the detailed database information
          #
          def databases_details_on(instance)
            databases_on(instance).collect do |db_summary|
              db_details = ::EasyType::Helpers::InstancesResults[db2("get database config for #{db_summary['Database name']}", :instance => instance).scan(/\s+(.*?)\s+=(.*)$/)]
              db_summary.merge(db_details)
            end
          end

          def tablespaces_on(instance, database)
            sql('select * from syscat.tablespaces', :instance => instance, :database => database, :parse => true)
          end

          def users_on(instance, database)
            query = <<-QUERY
              select authid as username, d.*
              from sysibmadm.authorizationids a
              left join syscat.dbauth d on d.grantee = a.authid
              where authidtype = 'U'
            QUERY
            sql(query, :instance => instance, :database => database, :parse => true)
          end

          def groups_on(instance, database)
            query = <<-QUERY
              select authid as groupname, d.*
              from sysibmadm.authorizationids a
              left join syscat.dbauth d on d.grantee = a.authid
              where authidtype = 'G'
            QUERY
            sql(query, :instance => instance, :database => database, :parse => true)
          end

          def schemas_on(instance, database)
            query = <<-QUERY
              select * from syscat.schemata
            QUERY
            sql(query, :instance => instance, :database => database, :parse => true)
          end

          def roles_on(instance, database)
            query = <<-QUERY
            select * from syscat.roles
            QUERY
            sql(query, :instance => instance, :database => database, :parse => true)
          end

          def grants_on(instance, database)
            query = <<-QUERY
            select * from syscat.tabauth
            QUERY
            sql(query, :instance => instance, :database => database, :parse => true)
          end
        end
      end
    end
  end
end
# rubocop: enable Style/ClassVars
